#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <getopt.h>

#include "config.h"
#include "measures.h"
#include "objFormat.h"
#include "objLevel.h"
#include "objSpeedo.h"
#include "objThermo.h"
#include "objLabel.h"
#include "error.h"

char Version [] = "XHardMon 1.0 (c)1999 F.COUTANT" ;

char ConfigFileName [FILENAME_MAX] = "" ;
char Title [80] = "Hardware Monitor" ;
char * PanelPixmapPath = NULL ;
char * DisplayName = NULL ;
int WinWidth = 0 ;
char DigitalColorName [20] = "white" ;
char AnalogGoodColorName [20] = "green" ;
char AnalogWarningColorName [20] = "yellow" ;
char AnalogAlertColorName [20] = "red" ;
char TextColorName [20] = "black" ;
char FontName [20] = "6x10" ;
int ArgCount = 0 ;
char ** ArgTab = NULL ;
char AppName [20] = "xhardmon" ;
char AppClass [20] = "HardMon" ;
char TempFmt [20] = "%2dC" ;
int UpdatePeriod=60 ;

extern void Config_Load (FILE * f)
{
    char c ;
    Object_Ptr obj ;
    char name [15] ;

    while (fscanf (f, " %c", & c) == 1)
    {
	ungetc (c, f) ;

	if (c == '#')
	{
	    fscanf (f, "%*[^\n]") ;
	}
	else
	if (fscanf (f, " %14[a-z] : ", name) != 1)
	{
	    error ("config: expected #... or <name>:...") ;
	}
	else
	if (strcmp (name, "title") == 0)
	{
	    if (fscanf (f, " \"%79[^\"]\" ;", Title) != 1)
	    {
		error ("config: expected title: \"...\"") ;
	    }
	}
	else
	if (strcmp (name, "font") == 0)
	{
	    if (fscanf (f, " \"%19[^\"]\" ;", FontName) != 1)
	    {
		error ("config: expected font: \"...\"") ;
	    }
	}
	else
	if (strcmp (name, "clippanel") == 0)
	{
	    if (fscanf (f, " %d ;", & WinWidth) != 1)
	    {
		error ("config: expected clip-panel: WINDOWWIDTH") ;
	    }
	}
	else
	if (strcmp (name, "colors") == 0)
	{
	    if (fscanf (f, " text = %19s digital = %19s analog = "
		"%19[a-z0-9] , %19[a-z0-9] , %19[a-z0-9] ;",
		TextColorName, DigitalColorName, AnalogGoodColorName,
		AnalogWarningColorName, AnalogAlertColorName) != 5)
	    {
		error ("config: expected font: \"...\"") ;
	    }
	}
	else
	if (strcmp (name, "panel") == 0)
	{
	    static char path [200] ;

	    if (fscanf (f, "\"%199[^\"]\" ;", path) != 1)
	    {
		error ("config: expected panel: \"<pixmap-path>\"") ;
	    }
	    else
	    {
		PanelPixmapPath = path ;
	    }
	}
	else
	if (strcmp (name, "dir") == 0)
	{
	    char path [200] ;

	    if (fscanf (f, "\"%199[^\"]\" ;", path) != 1)
	    {
		error ("config: expected panel: \"<pixmap-path>\"") ;
	    }
	    else
	    {
		Measure_SetBaseDir (path) ;
	    }
	}
	else
	if (strcmp (name, "measure") == 0)
	{
            Measure_Parse (Measure_Add (), f) ;
	}
	else
	if (strcmp (name, "format") == 0)
	{
	    char value [20] ;

	    if (fscanf (f, "%19[a-z0-9] ", value) != 1)
	    {
		error ("format: expected <name> ... ;") ;
	    }
	    else
	    {
		obj = Format_Create () ;
		Object_Parse (obj, Measure_Find (value), f) ;
		Object_Add (obj) ;
	    }
	}
	else
	if (strcmp (name, "level") == 0)
	{
	    char value [20] ;

	    if (fscanf (f, "%19[a-z0-9] ", value) != 1)
	    {
		error ("level: expected <name> ... ;") ;
	    }
	    else
	    {
		obj = Level_Create () ;
		Object_Parse (obj, Measure_Find (value), f) ;
		Object_Add (obj) ;
	    }
	}
	else
	if (strcmp (name, "speedo") == 0)
	{
	    char value [20] ;

	    if (fscanf (f, "%19[a-z0-9] ", value) != 1)
	    {
		error ("speedo: expected <name> ... ;") ;
	    }
	    else
	    {
		obj = Speedo_Create () ;
		Object_Parse (obj, Measure_Find (value), f) ;
		Object_Add (obj) ;
	    }
	}
	else
	if (strcmp (name, "thermo") == 0)
	{
	    char value [20] ;

	    if (fscanf (f, "%19[a-z0-9] ", value) != 1)
	    {
		error ("thermo: expected <name> ... ;") ;
	    }
	    else
	    {
		obj = Thermo_Create () ;
		Object_Parse (obj, Measure_Find (value), f) ;
		Object_Add (obj) ;
	    }
	}
	else
	if (strcmp (name, "text") == 0)
	{
	    obj = Label_Create () ;
	    Object_Parse (obj, -1, f) ;
	    Object_Add (obj) ;
	}
    }
}

static void usage (const char * prog)
{
    printf ("usage: %s [options...]\n"
	"where options are :\n"
	"    -f FILENAME    use configuration data from file FILENAME\n"
	"    -u N           update measures every N seconds\n"
	"    -v             show version\n"
	"    -h             this help text.\n"
	"", prog) ;
}

extern void InitConfig (void)
{
    int opt ;
    FILE * f ;

    while ((opt = getopt (ArgCount, ArgTab, "u:f:vh")) != EOF) switch (opt)
    {
	case 'u' :
	    UpdatePeriod = atoi (optarg) ;
	    break ;

	case 'f' :
	    strcpy (ConfigFileName, optarg) ;
	    break ;

	case 'v' :
	    printf ("%s\n", Version) ;
	    exit (0) ;
	    break ;

	case '?' :
	case 'h' :
	    usage (ArgTab [0]) ;
	    error ("program execution aborted") ;
	    break ;

	default :
	    break ;
    }

    if (ConfigFileName[0] == 0)
    {
	usage (ArgTab [0]) ;
	error ("no configuration file provided") ;
    }
    else
    if ((f = fopen (ConfigFileName, "r")) != NULL)
    {
	Config_Load (f) ;
	fclose (f) ;
    }
    else
    {
	error ("cannot open config file \"%s\"", ConfigFileName) ;
    }
}

