#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>

#include "events.h"
#include "display.h"
#include "win.h"
#include "config.h"
#include "measures.h"
#include "objects.h"
#include "refresh.h"

static XEvent AlarmEv ;

static void gotAlarm (int sig)
{
    signal (SIGALRM, gotAlarm) ;
    alarm (UpdatePeriod) ;

    XSendEvent (Dpy, Panel, False, 0, & AlarmEv) ;
    XFlush (Dpy) ;
}

extern void EventLoop (void)
{
    XEvent ev ;

    AlarmEv.type = Expose ;
    AlarmEv.xexpose.count = 0 ;
    /* dummy, not used */
    AlarmEv.xexpose.x = 0 ;
    AlarmEv.xexpose.y = 0 ;
    AlarmEv.xexpose.width = 0 ;
    AlarmEv.xexpose.height = 0 ;

    gotAlarm (0) ;

    while (1)
    {
	XNextEvent (Dpy, & ev) ;

	switch (ev.type)
	{
	    case Expose :
		if (ev.xexpose.count != 0) break ;
		if (ev.xany.send_event)
		{
		    Measure_UpdateAll () ;
		    Object_UpdateAll () ;
		}
		refresh () ;
		break ;

	    case ClientMessage :
	    {
		Atom subType ;
		const char * typeName ;

		subType = ev.xclient.message_type ;
		typeName = XGetAtomName (Dpy, subType) ;

		if (strcmp (typeName, "WM_PROTOCOLS") == 0)
		{
		    /* DELETE_WINDOW since we registered only this one */
		    exit (0) ;
		}
		else
		{
		    fprintf (stderr, "got unknown client message type=%s\n",
			typeName) ;
		}

		break ;
	    }

	    default :
		fprintf (stderr, "got unknown event type=%d\n",
		    (int)ev.type) ;
		break ;
	}
    }

    /* disable alarms */
    alarm (0) ;
    signal (SIGALRM, SIG_IGN) ;
}

