#!/bin/sh
set -e

error () {
    echo "$*" >&2
    exit 1
}

if [ $# -lt 3 ]
then
    error "usage: ./instconf.sh <BINDIR> <conf files>... <DESTDIR>"
fi

sensordir="/proc/sys/dev/sensors"
chipsfile="$sensordir/chips"
if [ ! -r "$chipsfile" ]
then
    error "$chipsfile is not readable. Is lm_sensors correctly loaded ?"
fi

chip=`awk '{print $2}' "$chipsfile" | head -n 1`
chipdir="$sensordir/$chip"
if [ ! -d "$chipdir" ]
then
    error "$chipdir does not exist or is not a directory." \
	"Is lm_sensors correctly functionning ?"
else
    echo "Monitoring chip found: $chip"
fi

BINDIR="$1"; shift
eval "DESTDIR=\$$#"

if [ ! -d "$DESTDIR" ]
then
    error "$DESTDIR is not a directory"
fi

while [ $# -gt 1 ]
do
    name=`basename "$1"`
    dest="$DESTDIR/$name"
    echo "$1 -> $dest"
    sed \
	-e "s,^#!.*xhardmon,#!${BINDIR}/xhardmon," \
	-e "s,dir:.*\",dir: \"${chipdir}\"," \
	< "$1" > "$dest"
    chmod 755 "$dest"
    shift
done

