#include <assert.h>
#include <stdio.h>
#include <string.h>

#include "measures.h"
#include "error.h"

#define MAXMES 12

typedef struct
{
    char name [11] ;
    char file [11] ;
    double mult ;
    double add ;
    double curVal ;
}
MeasureDef, * MeasureDefPtr ;

static MeasureDef Measures [MAXMES] ;
static int NbMeasures = 0 ;

static char BaseDir [FILENAME_MAX] ;
static char * FilePlace = 0 ;

extern void Measure_SetBaseDir (const char * dir)
{
    assert (dir != NULL) ;
    sprintf (BaseDir, "%.*s", (int)sizeof BaseDir - 2, dir) ;
    FilePlace = BaseDir + strlen (BaseDir) ;

    /* append a slash to be ready to add file name */
    if (FilePlace > BaseDir && FilePlace[-1] != '/')
    {
	*FilePlace++ = '/' ;
    }
}

extern int Measure_Add (void)
{
    int ret ;
    MeasureDefPtr m ;

    assert (NbMeasures < MAXMES) ;

    ret = NbMeasures++ ;
    m = Measures + ret ;
    m->name[0] = 0 ;
    m->file[0] = 0 ;
    m->curVal = 0.0L ;
    m->mult = 1.0L ;
    m->add = 0.0L ;

    return ret ;
}

extern void Measure_Parse (int index, FILE * f)
{
    MeasureDefPtr m ;
    char c ;

    assert (f != NULL) ;
    assert (0 <= index && index <= NbMeasures) ;

    m = Measures + index ;

    if (fscanf (f, "%10[a-z0-9] = %10[a-z0-9] %c", m->name, m->file, & c) != 3)
    {
        error ("measure: expected <name>=<file> [*REAL] [+REAL] ;") ;
    }

    if (c == '*')
    {
        if (fscanf (f, "%lf %c", & m->mult, & c) != 2)
        {
            error ("measure: expected *REAL [+REAL] ;") ;
        }
    }

    if (c == '+')
    {
        if (fscanf (f, "%lf %c", & m->add, & c) != 2)
        {
            error ("measure: expected +REAL ;") ;
        }
    }

    if (c != ';')
    {
        error ("measure: expected terminating ';'") ;
    }
}

extern int Measure_Find (const char * name)
{
    int i ;

    assert (name != NULL) ;

    for (i=0; i<NbMeasures; i++)
	if (strcmp (name, Measures[i].name) == 0)
	    return i ;

    error ("measure: '%s' not found", name) ;
    return -1 ;
}

extern double Measure_Get (int index)
{
    assert (0 <= index && index < NbMeasures) ;
    return Measures[index].curVal ;
}

static double rdValFromFile (const char * name)
{
    FILE * f ;
    double ret ;

    strcpy (FilePlace, name) ;
    if ((f = fopen (BaseDir, "r")) == NULL)
    {
	return 0.0L ;
    }
    else
    {
	double val ;

	/* read last value in file, return 0 if no value can be read */
	ret = 0.0L ;
	while (fscanf (f, "%lf", & val) == 1)
	{
	    ret = val ;
	}
    }

    fclose (f) ;
    return ret ;
}

extern void Measure_UpdateAll (void)
{
    MeasureDefPtr m ;
    int i ;
    for (i=0; i<NbMeasures; i++)
    {
        m = Measures + i ;
	m->curVal = rdValFromFile (m->file) * m->mult + m->add ;
    }
}

