#include <assert.h>
#include <string.h>

#include "objFormat.h"
#include "measures.h"
#include "display.h"
#include "win.h"
#include "gc.h"
#include "error.h"

#define MAXFMTS 20

static void update (Object_Ptr) ;
static void draw (Object_Ptr) ;
static void parse (Object_Ptr, int, FILE *) ;

static Object_Methods FormatMethods =
{
    update,
    draw,
    parse
} ;

typedef struct Object__
{
    Object_MethodsPtr methods ;
    int valIndex ;
    int x ;
    int y ;
    char fmt [20] ;
    char text [20] ;
}
Object ;

static Object Objects [MAXFMTS] ;
static int NbObjects = 0 ;

extern Object_Ptr Format_Create (void)
{
    Object_Ptr obj ;
    assert (NbObjects < MAXFMTS) ;
    obj = Objects + NbObjects++ ;

    obj->methods = & FormatMethods ;
    obj->valIndex = -1 ;
    obj->x = 0 ;
    obj->y = 0 ;
    obj->fmt[0] = 0 ;

    return obj ;
}

static void update (Object_Ptr obj)
{
    sprintf (obj->text, obj->fmt, Measure_Get (obj->valIndex)) ;
}

static void draw (Object_Ptr obj)
{
    XDrawString (Dpy, Panel, DigitalGC, obj->x, obj->y, obj->text,
	strlen (obj->text)) ;
}

static void parse (Object_Ptr obj, int valIndex, FILE * f)
{
    char c ;

    assert (obj != NULL) ;
    assert (obj->methods == & FormatMethods) ;

    obj->valIndex = valIndex ;

    while (fscanf (f, " %c", & c) == 1)
    {
	if (c == ';')
	{
	    /* finished */
	    break ;
	}
	else
	{
	    char name [4] ;
	    ungetc (c, f) ;

	    if (fscanf (f, "%3[a-z] = ", name) != 1)
	    {
		error ("format: expected <name>=...") ;
	    }
	    else
	    if (strcmp (name, "pos") == 0)
	    {
		if (fscanf (f, "%d , %d ", & obj->x, & obj->y) != 2)
		{
		    error ("format: expected pos=<integer>,<integer>") ;
		}
	    }
	    else
	    if (strcmp (name, "fmt") == 0)
	    {
		if (fscanf (f, "\"%19[^\"]\" ", obj->fmt) != 1)
		{
		    error ("format: expected fmt=\"<format>\"") ;
		}
	    }
	    else
	    {
		error ("format: unknown syntax '%s ='", name) ;
	    }
	}
    }

    if (c != ';')
    {
	error ("format: unexpected end-of-file") ;
    }
}

