#include <assert.h>
#include <string.h>

#include "objLabel.h"
#include "display.h"
#include "win.h"
#include "gc.h"
#include "error.h"

#define MAXLBLS 10

static void update (Object_Ptr) ;
static void draw (Object_Ptr) ;
static void parse (Object_Ptr, int, FILE *) ;

static Object_Methods LabelMethods =
{
    update,
    draw,
    parse
} ;

typedef struct Object__
{
    Object_MethodsPtr methods ;
    int x ;
    int y ;
    char text [10] ;
    int labelDrawn ;
}
Object ;

static Object Objects [MAXLBLS] ;
static int NbObjects = 0 ;

extern Object_Ptr Label_Create (void)
{
    Object_Ptr obj ;
    assert (NbObjects < MAXLBLS) ;
    obj = Objects + NbObjects++ ;

    obj->methods = & LabelMethods ;
    obj->x = 0 ;
    obj->y = 0 ;
    obj->text[0] = 0 ;
    obj->labelDrawn = 0 ;

    return obj ;
}

static void update (Object_Ptr obj)
{
    assert (obj != NULL) ;
    assert (obj->methods == & LabelMethods) ;

    if (obj->labelDrawn) return ;

    XDrawString (Dpy, EmptyPanel, TextGC, obj->x, obj->y, obj->text,
	strlen (obj->text)) ;
    obj->labelDrawn = 1 ;
}

static void draw (Object_Ptr obj)
{
    /* nothing */
}

static void parse (Object_Ptr obj, int valIndex, FILE * f)
{
    char c ;

    assert (obj != NULL) ;
    assert (obj->methods == & LabelMethods) ;

    while (fscanf (f, " %c", & c) == 1)
    {
	if (c == ';')
	{
	    /* finished */
	    break ;
	}
	else
	{
	    char name [5] ;
	    ungetc (c, f) ;

	    if (fscanf (f, "%4[a-z] = ", name) != 1)
	    {
		error ("label: expected <name>=...") ;
	    }
	    else
	    if (strcmp (name, "pos") == 0)
	    {
		if (fscanf (f, "%d , %d ", & obj->x, & obj->y) != 2)
		{
		    error ("label: expected pos=<integer>,<integer>") ;
		}
	    }
	    else
	    if (strcmp (name, "text") == 0)
	    {
		if (fscanf (f, "\"%9[^\"]\" ", obj->text) != 1)
		{
		    error ("label: expected text=\"<text>\"") ;
		}
	    }
	    else
	    {
		error ("label: unknown syntax '%s ='", name) ;
	    }
	}
    }

    if (c != ';')
    {
	error ("label: unexpected end-of-file") ;
    }
}

