#include "win.h"
#include "display.h"
#include "config.h"
#include "error.h"
#include "refresh.h"
#include "panel-empty.xpm"
#include <X11/xpm.h>

/*
XSetWMHints (dpy, win, XWMHints * in) ;
*/

Window Panel = 0 ;
Pixmap EmptyPanel = 0 ;

static int bgWidth = 0 ;
static int bgHeight = 0 ;

static void InitPixmap (void)
{
    if (EmptyPanel == 0)
    {
	int ret ;
	XpmAttributes attr ;

	attr.valuemask = 0 ;

	if (PanelPixmapPath == NULL)
	{
	    ret = XpmCreatePixmapFromData (
		Dpy, RootWindow (Dpy, Scr),
		panel_empty_xpm,
		& EmptyPanel, NULL, & attr) ;
	}
	else
	{
	    ret = XpmReadFileToPixmap (
		Dpy, RootWindow (Dpy, Scr),
		PanelPixmapPath,
		& EmptyPanel, NULL, & attr) ;
	}

	if (ret != XpmSuccess)
	{
	    error ("could not load XPM") ;
	}

	bgWidth = attr.width ;
	bgHeight = attr.height ;
    }
}

extern void InitMainWindow (void)
{
    if (Panel == 0)
    {
	XSizeHints szHint ;
	Atom wmProt ;
	XClassHint clsHint ;

	InitDisplay () ;
	InitPixmap () ;

	if (WinWidth <= 0 || WinWidth > bgWidth)
	{
	    WinWidth = bgWidth ;
	}

	clsHint.res_name = AppName ;
	clsHint.res_class = AppClass ;

	/* create window */
	Panel = XCreateSimpleWindow (
	    Dpy, RootWindow (Dpy, Scr),
	    0, 0, WinWidth, bgHeight, 0, BlackPixel (Dpy, Scr),
	    BlackPixel (Dpy, Scr)) ;
	/* set the previously loaded pixmap as background pixmap, so that */
	/* repainting is done by X */
	XSetWindowBackgroundPixmap (Dpy, Panel, EmptyPanel) ;
	/* sets initial size and size constraints (not resizable !) */
	/* and sets window name and application command line */
	szHint.flags = PSize | PMinSize | PMaxSize ;
	szHint.width = szHint.min_width = szHint.max_width = WinWidth ;
	szHint.height = szHint.min_height = szHint.max_height = bgHeight ;
	XSetStandardProperties (Dpy, Panel,
	    Title, Title, None, ArgTab, ArgCount, & szHint) ;
	XSetClassHint (Dpy, Panel, & clsHint) ;
	/* inform WM that we support DELETE protocol */
	wmProt = XInternAtom (Dpy, "WM_DELETE_WINDOW", False) ;
	XSetWMProtocols (Dpy, Panel, & wmProt, 1) ;
	/* we are interested in Expose and DestroyNotify events */
#if 0
	XSelectInput (Dpy, Panel, ExposureMask | StructureNotifyMask) ;
#else
	XSelectInput (Dpy, Panel, ExposureMask) ;
#endif
	/* map the window raised */
	XMapRaised (Dpy, Panel) ;
    }
}

